package com.gic.auth.service.outer.impl;

import java.util.ArrayList;
import java.util.List;

import com.gic.api.base.commons.Page;
import com.gic.auth.constant.GicVersionEnum;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.commons.util.PageHelperUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.constant.MenuTypeEnum;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;

@Service("menuApiService")
public class MenuApiServiceImpl implements MenuApiService {
    @Autowired
    private MenuService menuService;
    @Autowired
    private AuthCodeApiService authCodeApiService;

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenu(Integer userId, Integer enterpriseId, Integer currentMenu) {
        MenuListQO params = new MenuListQO();
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        List<TabSysMenu> menuList = this.menuService.listMenu(params);
        int level = 1;
        int parentId = 0;
        if (currentMenu != null) {
            TabSysMenu menu = this.menuService.getMenuById(currentMenu);
            parentId = currentMenu;
            level = menu.getLevel() + 1;
        }
        return ServiceResponse.success(this.treeMenu(menuList, level, parentId));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getSuperAdminMenu(Integer userId, Integer enterpriseId) {
        return this.getUserMenu(userId, enterpriseId, null);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTree(MenuListQO params) {
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        List<TabSysMenu> menuList = this.menuService.listMenu(params);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuRoleByParentId(Integer parentId) {
        TabSysMenu record = menuService.getMenuById(parentId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，页面不存在");
        }
        List<TabSysMenu> menuList = menuService.listMenuRoleByParentId(parentId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<Void> saveGICPage(MenuDTO menuDTO) {
        menuDTO.setProject(MenuProjectConstants.DEFAULT_PROJECT);
        return savePage(menuDTO, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateGICPage(MenuDTO menuDTO) {
        return updatePage(menuDTO, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<Void> deleteMenu(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuId(menuId);
        menuDTO.setStatus(0);
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveGICChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateGICChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuDTO.SaveGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppPage(MenuDTO menuDTO) {
        return updatePage(menuDTO, MenuDTO.UpdateAppPageValid.class);
    }

    @Override
    public ServiceResponse<Void> saveAppChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuDTO.EditAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> setSort(Integer id, Integer sort) {
        TabSysMenu record = menuService.getMenuById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (sort == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值不能为空");
        }
        menuService.setSort(record, sort);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveOperationItem(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.OPERATION.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveOperationItemValid.class);
    }

    @Override
    public ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuDTO.SaveOperationItemValid.class);
    }

    @Override
    public ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params) {
        com.github.pagehelper.Page page = menuService.pageMenuOperationItem(params);
        Page<MenuDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, MenuDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<MenuDTO> getDetailById(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(MenuDTO.class, record));
    }


    private ServiceResponse<Void> savePage(MenuDTO menuDTO, Class clazz) {
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        setMenuVersion(menuDTO);
        //sort
        int maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, menuDTO.getProject());
        menuDTO.setSort(maxSort + 1);
        //level
        menuDTO.setLevel(1);

        menuDTO.setParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID);
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        menuService.savePage(menuDTO);
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> saveChildPage(MenuDTO menuDTO, Class clazz) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //project
        menuDTO.setProject(record.getProject());
        setMenuVersion(menuDTO);
        //sort
        int maxSort = menuService.getMaxSortByParentId(menuDTO.getParentId(), menuDTO.getProject());
        menuDTO.setSort(maxSort + 1);
        //level
        menuDTO.setLevel(record.getLevel() + 1);
        //父级code
        menuDTO.setParentCode(record.getMenuCode());

        menuService.savePage(menuDTO);
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> updatePage(MenuDTO menuDTO, Class clazz) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getLevel().intValue() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非第一级页面");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        menuDTO.setProject(record.getProject());
        setMenuVersion(menuDTO);

        menuDTO.setMenuVersion("_" + menuDTO.getMenuVersion().replaceAll(",", "_") + "_");
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> updateChildPage(MenuDTO menuDTO, Class clazz) {
        TabSysMenu parent = menuService.getMenuById(menuDTO.getParentId());
        if (parent == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (parent.getLevel().intValue() >= record.getLevel().intValue()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID有误，层级在当前页面之下");
        }
        if (!parent.getProject().equals(record.getProject())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面和当前页面不在同一项目应用内");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        menuDTO.setProject(record.getProject());
        setMenuVersion(menuDTO);
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    private static void setMenuVersion(MenuDTO menuDTO) {
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            menuDTO.setMenuVersion("_" + menuDTO.getMenuVersion().replaceAll(",", "_") + "_");
            String[] menuVersionArr = menuDTO.getMenuVersion().split("_");
            StringBuilder result = new StringBuilder();
            for (String str : menuVersionArr) {
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                result.append(GicVersionEnum.getMessageByCode(Integer.parseInt(str))).append("/");
            }
            String menuVersionStr = result.toString();
            menuDTO.setMenuVersionName(menuVersionStr.substring(0, menuVersionStr.length() - 1));
        } else {
            menuDTO.setMenuVersion("_" + menuDTO.getMenuVersion().replaceAll(",", "_") + "_");
            menuDTO.setMenuVersionName(menuDTO.getMenuVersionName().replaceAll(",", "/"));
        }
    }

    private List<MenuDTO> treeMenu(List<TabSysMenu> menuList, int level, int parentId) {
        List<MenuDTO> list = null;
        if (level < 5) {
            for (TabSysMenu menu : menuList) {
                if (level == menu.getLevel() && menu.getIsShow().intValue() == 1
                        && menu.getParentId().intValue() == parentId) {
                    MenuDTO menuDTO = EntityUtil.changeEntityByJSON(MenuDTO.class, menu);
                    menuDTO.setChildren(this.treeMenu(menuList, menu.getLevel().intValue() + 1, menuDTO.getMenuId()));
                    if (list == null) {
                        list = new ArrayList<>();
                    }
                    list.add(menuDTO);
                }
            }
        }
        return list;
    }
}
